/*
 * chain_filter.cpp
 *
 *  Created on: 2011/01/27
 *      Author: shu
 */

#include <vector>
#include <math.h>
#include <algorithm>
#include "statistics.h"
#include "aligner.h"
#include "chain_filter.h"
#include "db.h"
#include "query.h"

using namespace std;

ChainFilter::ChainFilter(const AlignerParameters &parameters)
: parameters_(parameters), chains_(0)
{}

void ChainFilter::Filter(Query &query, Db &db, vector<Hit> &hits) {
  chains_.resize(query.GetSequenceLength());
  for (vector<Chain>::iterator it = chains_.begin(); it < chains_.end(); ++it) {
    it->query_end = 0;
  }
  sort(hits.begin(), hits.end(), DbStartComp());
  uint32_t length_query = query.GetSequenceLength();
  uint32_t n = 0;
  for (vector<Hit>::iterator it = hits.begin(); it != hits.end(); ++it) {
    uint32_t diagonal = length_query - 1 + it->start.db_offset - it->start.queries_offset;
    uint32_t offset = diagonal % length_query;
    if (chains_[offset].diagonal != diagonal || !Connected(query, db,
        chains_[offset].query_end, *it, query.GetUngappedExtendNominalCutoff())) {
      chains_[offset].query_end = it->end.queries_offset;
      chains_[offset].diagonal = length_query - 1 + it->start.db_offset - it->start.queries_offset;
      hits[n] = *it;
      ++n;
    } else {
      chains_[offset].query_end = it->end.queries_offset;
    }
  }
  hits.resize(n);
  
}


bool ChainFilter::Connected(Query &query, Db &db, uint32_t chain_query_end, Hit &hit, int cutoff) {
  if (hit.start.queries_offset < chain_query_end) {
    return true;
  }
  Code *queries_data = query.GetSequence();
  Code *db_data = db.GetConcatenatedSequence();
  uint32_t q_offset = hit.start.queries_offset;
  uint32_t d_offset = hit.start.db_offset;
  int score = 0;
  int c = -cutoff;
  uint32_t number_letters = parameters_.GetScoreMatrix().GetNumberLetters();
  vector<int> m = parameters_.GetScoreMatrix().GetMatrix();
  int *score_matrix = &(m[0]);

  do {
    --d_offset;
    --q_offset;
    if (db_data[d_offset] == AlignerParameters::kSequenceEnd
        || queries_data[q_offset] == AlignerParameters::kSequenceEnd || q_offset <= chain_query_end) {
      break;
    }
    Code d = db_data[d_offset];
    Code q = queries_data[q_offset];
    score += score_matrix[d * number_letters + q];
  } while (score > c);
  if (q_offset < chain_query_end) {
    return true;
  } else {
    return false;
  }
}
