#include <cstddef>
#include <cstdlib>
#include <cmath>

#include <algorithm>
#include <fstream>
#include <functional>
#include <iostream>
#include <iterator>
#include <vector>
#include <map>
#include <numeric>
#include <stdexcept>
#include <sstream>
#include <utility>

#include "kmer_counter.h"

KmerCounter::KmerCounter(const std::string& cs)
  : cs_(cs), m_(), char_type_size_()
{
  for (std::string::const_iterator i=cs.begin();
      i < cs.end(); ++i) {
    if (m_.find(*i) == m_.end()) {
      m_.insert(std::make_pair(*i, m_.size()));
    }
  }
  char_type_size_ = m_.size();
}

KmerCounter::~KmerCounter()
{
}

void KmerCounter::count(const std::string& s, std::size_t k, std::vector<unsigned int>& cnt) const
{
  std::string::const_iterator i(s.begin());
  std::string::const_iterator end_of_s(s.end() - k + 1);
  for (; i < end_of_s; ++i) {
    std::string::const_iterator end_of_kmer(i + k);
    std::size_t x = 0;
    for (std::string::const_iterator j = i;
        j < end_of_kmer; ++j) {
      std::map<char, unsigned int>::const_iterator mi = m_.find(*j);
      if (mi == m_.end()) {
        goto break_nested_loop;
      }
      x = x * char_type_size_ + mi->second;
    }
    ++cnt[x];
break_nested_loop:
    ;
  }
}

std::size_t KmerCounter::char_type_size() const
{
  return char_type_size_;
}

std::size_t KmerCounter::kmer_type_size(std::size_t k) const
{
  std::size_t x = 1;
  for (std::size_t i=0; i<k; ++i) {
    x *= char_type_size_;
  }
  return x;
}

std::size_t len_poly_n(const std::string& s)
{
  std::size_t x = 0;
  std::size_t xx = 1;
  char prev_c = s[0];
  for (std::string::const_iterator i = s.begin() + 1;
      i < s.end(); ++i) {
    if (*i != prev_c) {
      prev_c = *i;
      if (xx > x) {
        x = xx;
      }
      xx = 1;
    } else {
      ++xx;
    }
  }
  if (xx > x) {
    x = xx;
  }
  return x;
}
