/*
 * sa_index.cpp
 *
 *  Created on: 2011/02/21
 *      Author: shu
 */

#include <fstream>
#include <vector>
#include <stdint.h>
#include "encoder.h"
#include "common_parameters.h"
#include "sa_sequence_index.h"

SaSequenceIndex::SaSequenceIndex(const unsigned char *text, unsigned int length, const Encoder &encoder) {
  vector<Code> index_seq(length);
  Code min_regular_letter_code = encoder.GetMinRegularLetterCode();
  Code max_regular_letter_code = encoder.GetMaxRegularLetterCode();
  Code unknown_code = encoder.GetUnknownCode();
  //int count = 0;
  for (uint32_t i = 0; i < index_seq.size(); ++i) {
    Code c = text[i];
    if (min_regular_letter_code <= c && c <= max_regular_letter_code) {
      index_seq[i] = c;
      //count = 0;
    } else {
      index_seq[i] = unknown_code;
    }
  }
  sa_.Rebuild(&index_seq[0], index_seq.size());
}

SaSequenceIndex::SaSequenceIndex(ifstream &is)
:sa_(is)
{}

SaSequenceIndex::~SaSequenceIndex()
{}

void SaSequenceIndex::Search(unsigned int offset, unsigned int start, unsigned int end, unsigned char *start_character, unsigned char *end_character,
    unsigned int *start_list, unsigned int *end_list) {
  sa_.Search(offset, start, end, start_character, end_character, start_list, end_list);
}

unsigned int SaSequenceIndex::Locate(unsigned int position) {
  return sa_.Locate(position);
}

unsigned int SaSequenceIndex::GetMaxAlphabet() {
  return sa_.GetMaxAlphabet();
}

void SaSequenceIndex::Load(ifstream &is) {
  sa_.Load(is);
}
void SaSequenceIndex::Save(ofstream &os) {
  sa_.Save(os);
}
